#  Copyright (c) 2022 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path

from tests.plugins import PluginTestCase
from troubadix.helper import CURRENT_ENCODING
from troubadix.plugin import LinterError, LinterFix
from troubadix.plugins.script_version_and_last_modification_tags import (
    CheckScriptVersionAndLastModificationTags,
)


class CheckScriptVersionAndLastModificationTagsTestCase(PluginTestCase):
    def test_ok(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            '  script_version("2021-07-19T12:32:02+0000");\n'
            '  script_tag(name: "last_modification", value: "2021-07-19 '
            '12:32:02 +0000 (Mon, 19 Jul 2021)");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckScriptVersionAndLastModificationTags(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_exclude_inc_file(self):
        path = Path("some/file.inc")
        fake_context = self.create_file_plugin_context(nasl_file=path)
        plugin = CheckScriptVersionAndLastModificationTags(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_nok(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            '  script_version("2021 07-19T12:32:02+0000");\n'
            '  script_tag(name: "last_modification", value: "2021_07-19 '
            '12:32:02 +0000 (Mon, 19 Jul 2021)");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckScriptVersionAndLastModificationTags(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 2)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "VT is using a wrong script_version(); syntax.",
            results[0].message,
        )
        self.assertEqual(
            "VT is is using a wrong syntax for script_tag(name:"
            '"last_modification".',
            results[1].message,
        )

    def test_old_nok(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            '  script_version("$Revision: 12345 $");\n'
            '  script_tag(name: "last_modification", value: "$Date: 2021-07-19 '
            '12:32:02 +0000 (Mon, 19 Jul 2021) $");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckScriptVersionAndLastModificationTags(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 2)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "VT is using a wrong script_version(); syntax.",
            results[0].message,
        )
        self.assertEqual(
            "VT is is using a wrong syntax for script_tag(name:"
            '"last_modification".',
            results[1].message,
        )

    def test_missing_script_version(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            '  script_tag(name:"cvss_base", value:"4.0");\n'
            '  script_tag(name:"summary", value:"Foo Bar.");\n'
            '  script_tag(name:"solution_type", value:"VendorFix");\n'
            '  script_tag(name:"solution", value:"meh");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckScriptVersionAndLastModificationTags(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "VT is missing script_version();.",
            results[0].message,
        )

    def test_version_wrong_format(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            '  script_version("TTTTT07-19T12:32:02+0000");\n'
            '  script_tag(name: "last_modification", value: "2021-07-19 '
            '12:32:02 +0000 (Mon, 19 Jul 2021)");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckScriptVersionAndLastModificationTags(fake_context)

        results = list(plugin.run())

        self.assertEqual(1, len(results))
        self.assertEqual(
            results[0].message, "False or incorrectly formatted version."
        )

    def test_fix_last_modification_date(self):
        with self.create_directory() as testdir:
            nasl_file = testdir / "test.nasl"
            content = (
                '  script_version("12345");\n'
                '  script_tag(name: "last_modification", '
                'value: "2021/07/19");\n'
            )
            nasl_file.write_text(content, encoding=CURRENT_ENCODING)
            fake_context = self.create_file_plugin_context(
                nasl_file=nasl_file, file_content=content
            )

            plugin = CheckScriptVersionAndLastModificationTags(fake_context)

            results = list(plugin.run())

            self.assertEqual(len(results), 2)

            results = list(plugin.fix())

            self.assertEqual(len(results), 1)
            self.assertIsInstance(results[0], LinterFix)

            new_content = nasl_file.read_text(encoding=CURRENT_ENCODING)
            self.assertNotEqual(content, new_content)

    def test_modification_date_wrong_format(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            '  script_version("2021-07-19T12:32:02+0000");\n'
            '  script_tag(name: "last_modification", value: "2021-07-19 '
            '12:32:02 +0000 (Mon, 19 Jul 2021x");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckScriptVersionAndLastModificationTags(fake_context)

        results = list(plugin.run())

        self.assertEqual(1, len(results))
        self.assertEqual(
            "False or incorrectly formatted modification_date.",
            results[0].message,
        )

    def test_modification_date_differing_dates(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            '  script_version("2021-07-19T12:32:02+0000");\n'
            '  script_tag(name: "last_modification", value: "3021-07-19 '
            '12:32:02 +0000 (Mon, 19 Jul 2021)");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckScriptVersionAndLastModificationTags(fake_context)

        results = list(plugin.run())

        self.assertEqual(1, len(results))
        self.assertEqual(
            "The modification_date consists of two different dates.",
            results[0].message,
        )

    def test_modification_date_wrong_day(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            '  script_version("2021-07-19T12:32:02+0000");\n'
            '  script_tag(name: "last_modification", value: "2021-07-19 '
            '12:32:02 +0000 (Tue, 19 Jul 2021)");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckScriptVersionAndLastModificationTags(fake_context)

        results = list(plugin.run())

        self.assertEqual(1, len(results))
        self.assertEqual(
            "Wrong day of week. Please change it from 'Tue' to 'Mon'.",
            results[0].message,
        )



#  Copyright (c) 2022 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from typing import Iterator

from troubadix.plugin import FilePlugin, LinterError, LinterResult

PATTERN = re.compile(r"[\t ]+$")


class CheckTrailingSpacesTabs(FilePlugin):
    name = "check_trailing_spaces_tabs"

    def run(self) -> Iterator[LinterResult]:
        """This script checks if a VT is using one or more trailing whitespaces
         or tabs.

        Args:
            nasl_file: The VT that is going to be checked
            file_content: The content of the VT
            tag_pattern: A dictionary of regex patterns that are used to find
                    tags
            special_tag_pattern: A dictionary of regex patterns that are used
                    to find special tags

        """
        for line_number, line in enumerate(
            self.context.file_content.splitlines(), start=1
        ):
            if not PATTERN.search(line):
                continue

            yield LinterError(
                "The VT has one or more trailing spaces "
                f"and/or tabs in line {line_number}!",
                file=self.context.nasl_file,
                plugin=self.name,
            )



# Copyright (C) 2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from pathlib import Path
from typing import Iterator

from troubadix.helper.patterns import (
    SpecialScriptTag,
    get_special_script_tag_pattern,
)
from troubadix.plugin import FileContentPlugin, LinterError, LinterResult


class CheckValidOID(FileContentPlugin):
    name = "check_valid_oid"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:
        """Checks if a NASL script is using a valid script_oid() tag.

        Valid:
        script_oid("1.3.6.1.4.1.25623.1.0.12345");
        script_oid('1.3.6.1.4.1.25623.1.0.12345');

        Not valid:
        script_oid(""); (no OID included)
        script_oid(); (empty tag)
        script_oid("1.3.6.1.4.1.25623.1.12345"); (malformed)

        Args:
            nasl_file: The VT that is going to be checked
            file_content: The content of the nasl_file

        """
        if (
            nasl_file.suffix == ".inc"
            or "# troubadix: disable=template_nd_test_files_fps" in file_content
        ):
            return

        security_template = "Security Advisory"
        family_template = "Local Security Checks"
        is_using_reserved = "is using an OID that is reserved for"
        invalid_oid = "is using an invalid OID"

        oid_pattern = get_special_script_tag_pattern(SpecialScriptTag.OID)
        oid_match = oid_pattern.search(file_content)
        if oid_match is None or oid_match.group("oid") is None:
            yield LinterError(
                "No valid script_oid() call found",
                file=nasl_file,
                plugin=self.name,
            )
            return

        oid = oid_match.group("oid")

        if "1.3.6.1.4.1.25623.1." not in oid:
            yield LinterError(
                f"script_oid() {invalid_oid} '{str(oid)}'",
                file=nasl_file,
                plugin=self.name,
            )
            return

        # Vendor-specific OIDs
        if "1.3.6.1.4.1.25623.1.1." in oid:
            family_pattern = get_special_script_tag_pattern(
                SpecialScriptTag.FAMILY
            )
            family_match = family_pattern.search(file_content)
            if family_match is None or family_match.group("value") is None:
                yield LinterError(
                    "VT is missing a script family!",
                    file=nasl_file,
                    plugin=self.name,
                )
                return

            family = family_match.group("value")

            vendor_number_match = re.search(
                r"^1\.3\.6\.1\.4\.1\.25623\.1\.1\.([0-9]+)\.", oid
            )
            if (
                vendor_number_match is None
                or vendor_number_match.group(1) is None
            ):
                yield LinterError(
                    f"script_oid() {invalid_oid} '{str(oid)}' (last digits)",
                    file=nasl_file,
                    plugin=self.name,
                )
                return

            vendor_number = vendor_number_match.group(1)

            if vendor_number == "1":
                if family != f"Debian {family_template}":
                    yield LinterError(
                        f"script_oid() {is_using_reserved} Debian "
                        f"'{str(oid)}'",
                        file=nasl_file,
                        plugin=self.name,
                    )
                    return

            elif vendor_number == "2":
                if family != f"Huawei EulerOS {family_template}":
                    yield LinterError(
                        f"script_oid() {is_using_reserved} EulerOS "
                        f"'{str(oid)}'",
                        file=nasl_file,
                        plugin=self.name,
                    )
                    return

                # Fixed OID-scheme for (Huawei) Euler OS OIDs
                euler_sa_match = re.search(
                    r"^1\.3\.6\.1\.4\.1\.25623\.1\.1\.2\.20[0-4][0-9]\.[0-9]{"
                    r"4}$",
                    oid,
                )
                if euler_sa_match is None:
                    yield LinterError(
                        f"script_oid() {invalid_oid} '{str(oid)}' (EulerOS "
                        "pattern: 1.3.6.1.4.1.25623.1.1.2.[ADVISORY_YEAR]"
                        ".[ADVISORY_ID])",
                        file=nasl_file,
                        plugin=self.name,
                    )
                return

            elif vendor_number == "4":
                if family != f"SuSE {family_template}":
                    yield LinterError(
                        f"script_oid() {is_using_reserved} SUSE SLES "
                        f"'{str(oid)}'",
                        file=nasl_file,
                        plugin=self.name,
                    )
                    return

                # Fixed OID-scheme for SUSE SLES OS OIDs
                sles_sa_match = re.search(
                    r"^1\.3\.6\.1\.4\.1\.25623\.1\.1\.4\.20[0-4][0-9]\.[0-9]{"
                    r"4,5}\.[0-9]$",
                    oid,
                )
                if sles_sa_match is None:
                    yield LinterError(
                        f"script_oid() {invalid_oid} '{str(oid)}' (SLES "
                        f"pattern: 1.3.6.1.4.1.25623.1.1.4.[ADVISORY_YEAR]"
                        f".[ADVISORY_ID].[ADVISORY_REVISION])",
                        file=nasl_file,
                        plugin=self.name,
                    )
                return

            elif vendor_number == "5":
                if family != f"Amazon Linux {family_template}":
                    yield LinterError(
                        f"script_oid() {is_using_reserved} Amazon Linux "
                        f"'{str(oid)}'",
                        file=nasl_file,
                        plugin=self.name,
                    )
                    return

                amazon_sa_match = re.search(
                    r"^1\.3\.6\.1\.4\.1\.25623\.1\.1\.5\.20[0-4][0-9]\.[0-9]+$",
                    oid,
                )
                if amazon_sa_match is None:
                    yield LinterError(
                        f"script_oid() {invalid_oid} '{str(oid)}' (Amazon "
                        "pattern: 1.3.6.1.4.1.25623.1.1.5.[ADVISORY_YEAR]"
                        ".[ADVISORY_ID])",
                        file=nasl_file,
                        plugin=self.name,
                    )
                return

            elif vendor_number == "6":
                if family != f"Gentoo {family_template}":
                    yield LinterError(
                        f"script_oid() {is_using_reserved} Gentoo "
                        f"'{str(oid)}'",
                        file=nasl_file,
                        plugin=self.name,
                    )
                    return

            elif vendor_number == "7":
                if family != "FreeBSD Local Security Checks":
                    yield LinterError(
                        f"script_oid() {is_using_reserved} FreeBSD "
                        f"'{str(oid)}'",
                        file=nasl_file,
                        plugin=self.name,
                    )
                    return

            elif vendor_number == "8":
                if family != f"Oracle Linux {family_template}":
                    yield LinterError(
                        f"script_oid() {is_using_reserved} Oracle Linux "
                        f"'{str(oid)}'",
                        file=nasl_file,
                        plugin=self.name,
                    )
                    return

            elif vendor_number == "9":
                if family != f"Fedora {family_template}":
                    yield LinterError(
                        f"script_oid() {is_using_reserved} Fedora "
                        f"'{str(oid)}'",
                        file=nasl_file,
                        plugin=self.name,
                    )
                    return

            elif vendor_number == "10":
                if family != f"Mageia Linux {family_template}":
                    yield LinterError(
                        f"script_oid() {is_using_reserved} Mageia Linux "
                        f"'{str(oid)}'",
                        file=nasl_file,
                        plugin=self.name,
                    )
                    return

                mageia_sa_match = re.search(
                    r"^1\.3\.6\.1\.4\.1\.25623\.1\.1\.10\.20[0-4][0-9]\.["
                    r"0-9]{4}$",
                    oid,
                )
                if mageia_sa_match is None:
                    yield LinterError(
                        f"script_oid() {invalid_oid} '{str(oid)}' (Mageia "
                        "pattern: 1.3.6.1.4.1.25623.1.1.10.[ADVISORY_YEAR]"
                        ".[ADVISORY_ID])",
                        file=nasl_file,
                        plugin=self.name,
                    )
                return

            elif vendor_number == "11":
                if family != f"Red Hat {family_template}":
                    yield LinterError(
                        f"script_oid() {is_using_reserved} Red Hat "
                        f"'{str(oid)}'",
                        file=nasl_file,
                        plugin=self.name,
                    )
                    return

            elif vendor_number == "12":
                if family != f"Ubuntu {family_template}":
                    yield LinterError(
                        f"script_oid() {is_using_reserved} Ubuntu "
                        f"'{str(oid)}'",
                        file=nasl_file,
                        plugin=self.name,
                    )
                    return

            elif vendor_number == "13":
                if family != f"Slackware {family_template}":
                    yield LinterError(
                        f"script_oid() {is_using_reserved} Slackware "
                        f"'{str(oid)}'"
                    )
                    return

                slackware_sa_match = re.search(
                    r"^1\.3\.6\.1\.4\.1\.25623\.1\.1\.13\.20[0-4][0-9]\.[0-9]"
                    r"{3,4}\.[0-9]{2}$",
                    oid,
                )
                if slackware_sa_match is None:
                    yield LinterError(
                        f"script_oid() {invalid_oid} '{str(oid)}' (Slackware "
                        "pattern: 1.3.6.1.4.1.25623.1.1.13.[ADVISORY_YEAR]"
                        ".[ADVISORY_ID].[ADVISORY_REVISION])"
                    )
                return

            elif vendor_number == "14":
                if family != f"Rocky Linux {family_template}":
                    yield LinterError(
                        f"script_oid() {is_using_reserved} Rocky Linux "
                        f"'{str(oid)}'",
                        file=nasl_file,
                        plugin=self.name,
                    )
                    return

            elif vendor_number == "15":
                if family != f"AlmaLinux {family_template}":
                    yield LinterError(
                        f"script_oid() {is_using_reserved} AlmaLinux "
                        f"'{str(oid)}'",
                        file=nasl_file,
                        plugin=self.name,
                    )
                    return

            else:
                yield LinterError(
                    f"script_oid() {invalid_oid} '{str(oid)}' (Vendor OID "
                    "with unknown Vendor-Prefix)",
                    file=nasl_file,
                    plugin=self.name,
                )
                return

            return

        # product-specific OIDs
        if "1.3.6.1.4.1.25623.1.2." in oid:
            name_patter = get_special_script_tag_pattern(SpecialScriptTag.NAME)
            name_match = name_patter.search(file_content)
            if not name_match or not name_match.group("value"):
                yield LinterError(
                    "VT is missing a script name!",
                    file=nasl_file,
                    plugin=self.name,
                )
                return

            name = name_match.group("value")

            # Fixed OID-scheme for Mozilla Firefox OIDs
            if "1.3.6.1.4.1.25623.1.2.1." in oid:
                if not name.startswith(f"Mozilla Firefox {security_template}"):
                    yield LinterError(
                        f"script_oid() {is_using_reserved} 'Firefox' ("
                        f"{str(oid)})",
                        file=nasl_file,
                        plugin=self.name,
                    )
                    return

                firefox_sa_match = re.search(
                    r"^1\.3\.6\.1\.4\.1\.25623\.1\.2\.1\.20[1-4][0-9]\.[0-9]{"
                    r"2,3}$",
                    oid,
                )
                if not firefox_sa_match:
                    yield LinterError(
                        f"script_oid() {invalid_oid} '{str(oid)}' "
                        "(Firefox pattern: 1.3.6.1.4.1.25623.1.2.1."
                        "[ADVISORY_YEAR].[ADVISORY_ID])",
                        file=nasl_file,
                        plugin=self.name,
                    )
                    return

                return

        oid_digit_match = re.search(
            r"^1\.3\.6\.1\.4\.1\.25623\.1\.0\.([0-9]+)", oid
        )
        if oid_digit_match is None or oid_digit_match.group(1) is None:
            yield LinterError(
                f"script_oid() {invalid_oid} '{str(oid)}' (last digits)",
                file=nasl_file,
                plugin=self.name,
            )
            return

        exceptions = [
            "ossim_server_detect.nasl",
            "gsf/2018/vmware/gb_vmware_fusion_vmxnet3_"
            "stack_memory_usage_vuln_macosx.nasl",
            "enterprise/2018/vmware/gb_vmware_fusion_vmxnet3_"
            "stack_memory_usage_vuln_macosx.nasl",
            "2008/asterisk_sdp_header_overflow.nasl",
            "2008/cisco_ios_ftp_server_auth_bypass.nasl",
            "2008/qk_smtp_server_dos.nasl",
            "2008/asterisk_pbx_guest_access_enabled.nasl",
            "2008/asterisk_null_pointer_dereference.nasl",
            "2008/goaheadwebserver_source_disclosure.nasl",
            "2011/secpod_ibm_lotus_domino_rpc_auth_dos_vuln.nasl",
            "2011/secpod_cubecart_mult_xss_and_sql_inj_vuln.nasl",
            "2016/gb_adobe_air_mult_vuln_feb16_macosx.nasl",
            "attic/gb_cybozu_garoon_mult_vuln_aug16.nasl",
            "2017/gb_openssh_mult_vuln_jan17_lin.nasl",
            "2017/gb_xenserver_ctx219378.nasl",
        ]
        # nb: Those are using invalid OID ranges but are already in
        # the feed since longer time and can't be fixed / changed.
        if any(e in str(nasl_file) for e in exceptions):
            return

        oid_digit = int(oid_digit_match.group(1))

        if (
            (10000 <= oid_digit <= 29999)
            or (50000 <= oid_digit <= 118999)
            or (120000 <= oid_digit <= 128999)
            or (130000 <= oid_digit <= 169999)
            or (170000 <= oid_digit <= 179999)
            or (200000 <= oid_digit <= 209999)
            or (700000 <= oid_digit <= 919999)
            or (1020000 <= oid_digit <= 1029999)
        ):
            return

        if 300000 <= oid_digit <= 309999:
            yield LinterError(
                f"script_oid() {invalid_oid} '{str(oid)}' (reserved OID "
                "range not part of the official Feed)",
                file=nasl_file,
                plugin=self.name,
            )
            return

        yield LinterError(
            f"script_oid() {invalid_oid} "
            f"'{str(oid)}' (unassigned OID range)",
            file=nasl_file,
            plugin=self.name,
        )



# Copyright (C) 2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path

from tests.plugins import PluginTestCase
from troubadix.plugin import LinterError
from troubadix.plugins.script_xref_url import CheckScriptXrefUrl


class CheckScriptXrefUrlTestCase(PluginTestCase):
    path = Path("some/file.nasl")

    def test_ok(self):
        content = (
            '  script_xref(name:"URL", value:"http://www.example.com");\n'
            # pylint: disable=line-too-long
            # Various cases from https://github.com/python-validators/validators/issues/296
            '  script_xref(name:"URL", value:"https://launchpad.support.sap.com/#/notes/2718993");\n'
            '  script_xref(name:"URL", value:"https://forums.livezilla.net/index.php?/topic/10983-fg-vd-19-086-livezilla-server-is-vulnerable-to-sql-injection-ii/");\n'
            '  script_xref(name:"URL", value:"http://www.brocade.com/en/backend-content/pdf-page.html?/content/dam/common/documents/content-types/security-bulletin/brocade-security-advisory-2016-168.pdf");\n'
            '  script_xref(name:"URL", value:"https://groups.google.com/forum/?fromgroups#!topic/rubyonrails-security/8SA-M3as7A8");\n'
            '  script_xref(name:"URL", value:"https://forum.bitdefender.com/index.php?/topic/75470-doubleagent/");\n'
            '  script_xref(name:"URL", value:"https://www.smartftp.com/forums/index.php?/topic/16425-smartftp-client-40-change-log/");\n'
            '  script_xref(name:"URL", value:"https://exchange.xforce.ibmcloud.com/#/vulnerabilities/100912");\n'
            '  script_xref(name:"URL", value:"https://support.microsoft.com/en-us/lifecycle#gp/Microsoft-Internet-Explorer");\n'
            '  script_xref(name:"URL", value:"https://www.watchguard.com/support/release-notes/fireware/12/en-US/EN_ReleaseNotes_Fireware_12_5_9/index.html#Fireware/en-US/resolved_issues.html");\n'
            '  script_xref(name:"URL", value:"https://code.wireshark.org/review/#/c/25660/");\n'
            '  script_xref(name:"URL", value:"https://groups.google.com/forum/#!topic/kubernetes-announce/yBrFf5nmvfI");\n'
            '  script_xref(name:"URL", value:"https://issues.sonatype.org/plugins/servlet/mobile#issue/NEXUS-16870");\n'
            '  script_xref(name:"URL", value:"http://forums.livezilla.net/index.php?/topic/163-livezilla-changelog/");\n'
            '  script_xref(name:"URL", value:"https://www.watchguard.com/support/release-notes/fireware/11/en-US/EN_ReleaseNotes_Fireware_11_12_1/index.html#Fireware/en-US/resolved_issues.html%3FTocPath%3D_____13");\n'
            '  script_xref(name:"URL", value:"https://review.typo3.org/#/c/37013");\n'
            '  script_xref(name:"URL", value:"https://forums.malwarebytes.org/index.php?/topic/158251-malwarebytes-anti-exploit-hall-of-fame/");\n'
            '  script_xref(name:"URL", value:"http://speedtouch.sourceforge.io/index.php?/news.en.html");\n'
            '  script_xref(name:"URL", value:"https://support.k7computing.com/index.php?/Knowledgebase/Article/View/173/41/advisory-issued-on-6th-november-2017");\n'
            '  script_xref(name:"URL", value:"http://support.novell.com/cgi-bin/search/searchtid.cgi?/10077872.htm");\n'
            # pylint: enable=line-too-long
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=self.path, file_content=content
        )
        plugin = CheckScriptXrefUrl(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_exclude_inc_file(self):
        path = Path("some/file.inc")
        fake_context = self.create_file_plugin_context(nasl_file=path)
        plugin = CheckScriptXrefUrl(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_invalid_url(self):
        content = '  script_xref(name:"URL", value:"www.example.com");\n'
        fake_context = self.create_file_plugin_context(
            nasl_file=self.path, file_content=content
        )
        plugin = CheckScriptXrefUrl(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            'script_xref(name:"URL", value:"www.example.com");: Invalid URL'
            " value",
            results[0].message,
        )



# SPDX-License-Identifier: GPL-3.0-or-later
# SPDX-FileCopyrightText: 2024 Greenbone AG

from pathlib import Path

from tests.plugins import PluginTestCase
from troubadix.plugin import LinterError
from troubadix.plugins.multiple_re_parameters import CheckMultipleReParameters


class CheckDuplicateReTestCase(PluginTestCase):
    def test_ok(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            '  script_mandatory_keys("foo/bar", re:"foo=1.2.3");\n'
            '#  script_mandatory_keys("foo/bar", re:"foo=1.2.3");\n'
            '  script_mandatory_keys("bar/foo");\n'
            "  line with no script_mandatory_keys pattern\n"
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, lines=content.splitlines()
        )
        plugin = CheckMultipleReParameters(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_fail(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            '  script_mandatory_keys("foo/bar", re:"foo=1.2.3");\n'
            '  script_mandatory_keys("bar/foo", re:"bar=3.2.1");\n'
            '#  script_mandatory_keys("bar/foo", re:"bar=3.2.1");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, lines=content.splitlines()
        )
        plugin = CheckMultipleReParameters(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "The re parameter of script_mandatory_keys can only "
            "be defined once, but was found 2 times",
            results[0].message,
        )



# SPDX-License-Identifier: GPL-3.0-or-later
# SPDX-FileCopyrightText: 2024 Greenbone AG

import re
from pathlib import Path
from typing import Iterable, Iterator

from troubadix.helper.patterns import (
    SpecialScriptTag,
    get_special_script_tag_pattern,
)
from troubadix.plugin import LineContentPlugin, LinterError, LinterResult

RE_PATTERN = re.compile(r"re\s*:")

MANDATORY_KEYS_PATTERN = get_special_script_tag_pattern(
    SpecialScriptTag.MANDATORY_KEYS
)


class CheckMultipleReParameters(LineContentPlugin):
    """This step checks if a VT
    has multiple mandatory_script_key re parameters
    """

    name = "check_multiple_re_parameters"

    def check_lines(
        self, nasl_file: Path, lines: Iterable[str]
    ) -> Iterator[LinterResult]:

        if self.context.nasl_file.suffix == ".inc":
            return

        re_pattern_count = 0
        for line in lines:
            if not (match := MANDATORY_KEYS_PATTERN.search(line)):
                continue
            if re.match(r"^\s*#", line):
                continue
            re_pattern_count += len(RE_PATTERN.findall(match.group("value")))

        if re_pattern_count > 1:
            yield LinterError(
                f"The re parameter of script_mandatory_keys can only "
                f"be defined once, but was found {re_pattern_count} times",
                file=nasl_file,
                plugin=self.name,
            )



#  Copyright (c) 2024 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path

from tests.plugins import PluginTestCase
from troubadix.plugin import LinterError, LinterWarning
from troubadix.plugins.overlong_description_lines import (
    CheckOverlongDescriptionLines,
)


class CheckOverlongDescriptionLinesTestCase(PluginTestCase):
    def test_ok(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            "ignored line that is not part of description"
            "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
            "if (description)\n"
            "{\n"
            '  script_version("2021-09-02T14:01:33+0000");\n'
            '  script_name("name is ignored xxxxxxxxxxxxxxxxxxxxxxxxx'
            'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx");\n'
            '  script_xref(name:"xref as well", value:"xxxxxxxxxxxxxx'
            'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx");\n'
            '  script_add_preference(name:"script_add_preference as well '
            'xxxxxxxxxxxxxxxxxxxxxxx", type:"checkbox", value:"no", id:1);\n'
            '  script_add_preference(type:"password", value:"", id:2, name:"'
            'Another variant of script_add_preference xxxxxxxxxxxxxxxxxxxx");\n'
            "  exit(0);\n"
            "}\n"
            "ignored line that is not part of description"
            "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckOverlongDescriptionLines(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_line_too_long(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            "if (description)\n"
            "{\n"
            "  too long line xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
            "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
            '  script_version("2021-09-02T14:01:33+0000");\n'
            "  exit(0);\n"
            "}\n"
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckOverlongDescriptionLines(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterWarning)
        self.assertEqual(
            "Line 3 is too long with 102 characters. Max 100",
            results[0].message,
        )

    def test_no_description_start_found(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            "{\n"
            "  ignored too long line xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
            "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
            '  script_version("2021-09-02T14:01:33+0000");\n'
            "  exit(0);\n"
            "}\n"
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckOverlongDescriptionLines(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "Check failed. Unable to find start of description block",
            results[0].message,
        )

    def test_no_description_end_found(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            "if (description)\n"
            "{\n"
            "  ignored too long line xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
            "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
            '  script_version("2021-09-02T14:01:33+0000");\n'
            "}\n"
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckOverlongDescriptionLines(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "Check failed. Unable to find end of description block",
            results[0].message,
        )



# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


import re
from argparse import ArgumentParser, Namespace
from pathlib import Path
from re import Pattern
from typing import List

from git.repo.base import Repo

DEFAULT_IGNORED_LINESTARTS = ["diff ", "index ", "--- ", "+++ ", "@@ "]


def ensure_pattern(argument: str) -> Pattern:
    return re.compile(argument)


def parse_arguments() -> Namespace:
    argument_parser = ArgumentParser(
        "Tool to check that the target rev only has diffs conforming to "
        "the given patterns regarding the source rev"
    )

    argument_parser.add_argument(
        "-d",
        "--directory",
        default=Path.cwd(),
        type=Path,
        help="The directory the repository to check is located in. "
        "Defaults to 'pwd'",
    )

    ignored_linestart_group = argument_parser.add_mutually_exclusive_group()

    ignored_linestart_group.add_argument(
        "-i",
        "--ignored-linestarts",
        dest="ignored_linestarts",
        nargs="*",
        type=str,
        default=DEFAULT_IGNORED_LINESTARTS,
        help="A list of line starts which will make the line be ignored. "
        "Default: %(default)s",
    )

    ignored_linestart_group.add_argument(
        "--ignored-linestart-file",
        type=read_ignored_linestarts,
        dest="ignored_linestarts",
        default=DEFAULT_IGNORED_LINESTARTS,
        help="The file containing a list of line starts, "
        "separated by newlines, which will make the line be ignored. "
        "Default values used: %(default)s",
    )

    pattern_group = argument_parser.add_mutually_exclusive_group(required=True)

    pattern_group.add_argument(
        "-p",
        "--patterns",
        type=ensure_pattern,
        nargs="*",
        help="The list of patterns to check the diff for",
    )

    pattern_group.add_argument(
        "--pattern-file",
        type=Path,
        help="The file containing the list of patterns, separated by newlines, "
        "to check the diff for",
    )

    argument_parser.add_argument(
        "-s", "--source", type=str, required=True, help="The upstream rev"
    )

    argument_parser.add_argument(
        "-t", "--target", type=str, required=True, help="The downstream rev"
    )

    return argument_parser.parse_args()


def check_diff_line_starts_with_ignored_linestart(
    line: str, ignored_linestarts: List[str]
) -> bool:
    return any(
        linestart
        for linestart in ignored_linestarts
        if line.startswith(linestart)
    )


def check_diff_line_matches_pattern(line: str, patterns: List[Pattern]) -> bool:
    return any(pattern for pattern in patterns if pattern.match(line))


def check_diff(
    lines: List[str], ignored_linestarts: List[str], patterns: List[Pattern]
) -> List[str]:
    return [
        line
        for line in lines
        if not check_diff_line_starts_with_ignored_linestart(
            line, ignored_linestarts
        )
        and not check_diff_line_matches_pattern(line, patterns)
    ]


def read_ignored_linestarts(path: Path) -> List[str]:
    if not path:
        return None

    with open(path, "r", encoding="UTF-8") as file:
        return [line.removesuffix("\n") for line in file.readlines()]


def read_patterns(path: Path) -> List[Pattern]:
    with open(path, "r", encoding="UTF-8") as file:
        return [
            re.compile(pattern.removesuffix("\n"))
            for pattern in file.readlines()
        ]


def main() -> int:
    arguments = parse_arguments()

    patterns = arguments.patterns or read_patterns(arguments.pattern_file)

    repo = Repo(arguments.directory)

    target = repo.commit(arguments.target)

    merge_base = repo.merge_base(repo.commit(arguments.source), target)[0]

    diff = repo.git.diff(target, merge_base, unified=0)

    result = check_diff(
        diff.splitlines(), arguments.ignored_linestarts, patterns
    )

    if result:
        print("The following lines don't match the any pattern:")
        print("\n".join(result))

        return 1

    return 0


if __name__ == "__main__":
    main()



#  Copyright (c) 2022 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
from pathlib import Path
from typing import Iterator

from troubadix.helper.patterns import ScriptTag, get_script_tag_pattern
from troubadix.plugin import FileContentPlugin, LinterError, LinterResult

SECURITY_MESSAGE_IMPLEMENTATIONS = [
    "security_message",
    "http_check_remote_code",
    "citrix_xenserver_check_report_is_vulnerable",
    "citrix_xenserver_report_missing_patch",
]


def _file_contains_security_message(file_content: str) -> bool:
    """Checks wether a VT content contains a call to security_message
    or any function known to implement it

    Args:
        file_content (str): The content of the VT
    """
    return any(
        security_message in file_content
        for security_message in SECURITY_MESSAGE_IMPLEMENTATIONS
    )


class CheckSecurityMessages(FileContentPlugin):
    name = "check_security_messages"

    def _check_security_message_present(
        self, nasl_file: Path, file_content: str
    ) -> Iterator[LinterResult]:
        """Checks that the VT does have a
        security_message or implementing function call

        Args:
            nasl_file (Path): The VTs path
            file_content (str): The content of the VT
        """
        deprecated_pattern = get_script_tag_pattern(
            script_tag=ScriptTag.DEPRECATED
        )
        if deprecated_pattern.search(file_content):
            return

        if not _file_contains_security_message(file_content):
            yield LinterError(
                "VT is missing a security_message or implementing"
                " function in a VT with severity",
                file=nasl_file,
                plugin=self.name,
            )

    def _check_security_message_absent(
        self, nasl_file: Path, file_content: str
    ) -> Iterator[LinterResult]:
        """Checks that the VT does not have a
        security_message or implementing function call

        Args:
            nasl_file (Path): The VTs path
            file_content (str): The content of the VT
        """
        # Policy VTs might use both, security_message and log_message
        if (
            "Policy/" in str(nasl_file)
            or "PCIDSS/" in str(nasl_file)
            or "GSHB/" in str(nasl_file)
        ):
            return

        if _file_contains_security_message(file_content):
            yield LinterError(
                "VT is using a security_message or implementing"
                " function in a VT without severity",
                file=nasl_file,
                plugin=self.name,
            )

    def _determinate_security_message_by_severity(
        self, file_content: str
    ) -> bool:
        """Determinates wether a VT requires a
        security_message or implementing function
        call

        Args:
            file_content (str): The content of the VT
        """
        cvss_base_pattern = get_script_tag_pattern(ScriptTag.CVSS_BASE)
        cvss_detect = cvss_base_pattern.search(file_content)

        return cvss_detect and cvss_detect.group("value") != "0.0"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:
        """This script checks the passed VT if it is using a security_message
        and having no severity (CVSS score) assigned or has a severity assigned
        but does not call security_message or an implementing function

        Args:
            nasl_file: The VT that is going to be checked
            file_content: The content of the VT

        """
        if nasl_file.suffix == ".inc":
            return

        security_message_required = (
            self._determinate_security_message_by_severity(file_content)
        )

        if security_message_required:
            yield from self._check_security_message_present(
                nasl_file, file_content
            )
        else:
            yield from self._check_security_message_absent(
                nasl_file, file_content
            )



#  Copyright (c) 2024 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from pathlib import Path
from typing import Iterator

from troubadix.plugin import (
    FileContentPlugin,
    LinterError,
    LinterResult,
    LinterWarning,
)

DESCRIPTION_START_PATTERN = re.compile(r"if\s*\(\s*description\s*\)")
DESCRIPTION_END_PATTERN = re.compile(r"exit\(0\)")
IGNORE_TAGS = [
    "script_name",
    "script_xref",
    "script_add_preference",
]


class CheckOverlongDescriptionLines(FileContentPlugin):
    """This step checks if a VT contains lines in the description block
    that are longer than 100 characters.
    """

    name = "check_overlong_description_lines"

    def check_content(
        self, nasl_file: Path, file_content: str
    ) -> Iterator[LinterResult]:
        if nasl_file.suffix == ".inc":
            return

        if not (start_match := DESCRIPTION_START_PATTERN.search(file_content)):
            yield LinterError(
                "Check failed. Unable to find start of description block",
                plugin=self.name,
                file=nasl_file,
            )
        if not (end_match := DESCRIPTION_END_PATTERN.search(file_content)):
            yield LinterError(
                "Check failed. Unable to find end of description block",
                plugin=self.name,
                file=nasl_file,
            )

        if not start_match or not end_match:
            return

        line_offset = file_content[: start_match.start()].count("\n")
        description_block = file_content[start_match.start() : end_match.end()]
        lines = description_block.splitlines()
        for i, line in enumerate(lines, 1 + line_offset):
            if len(line) > 100:
                if any(tag in line for tag in IGNORE_TAGS):
                    continue

                yield LinterWarning(
                    f"Line {i} is too long"
                    f" with {len(line)} characters. "
                    f"Max 100",
                    plugin=self.name,
                    file=nasl_file,
                    line=i,
                )



# Copyright (C) 2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import re
import subprocess
import sys
from argparse import ArgumentParser, Namespace
from pathlib import Path
from typing import Iterable

from troubadix.standalone_plugins.common import git


def file_type(string: str) -> Path:
    file_path = Path(string)
    if not file_path.is_file():
        raise ValueError(f"{string} is not a file.")
    return file_path


def parse_args(args: Iterable[str]) -> Namespace:
    parser = ArgumentParser(
        description="Check for changed oid",
    )
    parser.add_argument(
        "-c",
        "--commit_range",
        type=str,
        required=True,
        help=(
            "Git commit range to check e.g "
            "2c87f4b6062804231fd508411510ca07fd270380^..HEAD or"
            "YOUR_BRANCH..main"
        ),
    )
    parser.add_argument(
        "-f",
        "--files",
        nargs="+",
        type=file_type,
        default=[],
        help=(
            "List of files to diff. "
            "If empty use all files added or modified in the commit range."
        ),
    )
    return parser.parse_args(args=args)


def check_oid(args: Namespace) -> bool:
    """The script checks (via git diff) if the passed VT has changed the
    OID in the following tag:

    - script_oid("1.2.3");

    This is only allowed in rare cases (e.g. a single VT was split into
    two VTs).
    """

    if not args.files:
        args.files += [
            Path(f)
            for f in git(
                "diff", "--name-only", "--diff-filter=d", args.commit_range
            ).splitlines()
        ]

    rcode = False
    for nasl_file in args.files:
        if nasl_file.suffix != ".nasl" or not nasl_file.exists():
            continue

        print(f"Check file {nasl_file}")
        text = git(
            "-c",
            "color.status=false",
            "--no-pager",
            "diff",
            args.commit_range,
            nasl_file,
        )

        oid_added = re.search(
            r'^\+\s*script_oid\s*\(\s*["\'](?P<oid>[0-9.]+)["\']\s*\)\s*;',
            text,
            re.MULTILINE,
        )
        if not oid_added or not oid_added.group("oid"):
            continue

        oid_removed = re.search(
            r'^-\s*script_oid\s*\(\s*["\'](?P<oid>[0-9.]+)["\']\s*\)\s*;',
            text,
            re.MULTILINE,
        )
        if not oid_removed or not oid_removed.group("oid"):
            continue

        if oid_added.group("oid") != oid_removed.group("oid"):
            print(
                f"OID of VT {nasl_file} was changed. This is only allowed in "
                f"rare cases (e.g. a duplicate OID got fixed or a single VT "
                f"was split into two VTs)."
                f"\nOID NEW: {oid_added.group('oid')}"
                f"\nOID OLD: {oid_removed.group('oid')}",
                file=sys.stderr,
            )
            rcode = True
    return rcode


def main() -> int:
    args = sys.argv[1:]

    try:
        git_base = git("rev-parse", "--show-toplevel")
        os.chdir(git_base.rstrip("\n"))
    except subprocess.SubprocessError:
        print(
            "Your current working directory doesn't belong to a git repository"
        )
        return 1

    if check_oid(parse_args(args)):
        return 2

    return 0


if __name__ == "__main__":
    sys.exit(main())



#  Copyright (c) 2022 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
import re
from pathlib import Path
from typing import Iterator

from troubadix.helper import ScriptTag
from troubadix.helper.patterns import get_script_tag_pattern
from troubadix.plugin import (
    FileContentPlugin,
    LinterError,
    LinterResult,
    LinterWarning,
)


class CheckLogMessages(FileContentPlugin):
    name = "check_log_messages"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:
        """This script checks the passed VT if it is using a log_message and
            having a severity (CVSS score) assigned which is an error /
            debugging leftover in most cases.

        Args:
            nasl_file: The VT that is going to be checked
            file_content: The content of the VT

        """
        log_match = re.search(
            r"log_message\s*\([\s\n]*\)\s*(;|;\s*(\n|#))",
            file_content,
            re.MULTILINE,
        )
        if log_match:
            yield LinterError(
                "The VT is using an empty log_message() function",
                file=nasl_file,
                plugin=self.name,
            )

        if nasl_file.suffix == ".inc":
            return

        # Policy VTs might use both, security_message and log_message
        if "Policy/" in str(nasl_file):
            return

        # don't need to check detection scripts since they are for sure using
        # a log_message. all detection scripts have a cvss of 0.0
        cvss_pattern = get_script_tag_pattern(ScriptTag.CVSS_BASE)
        cvss_detect = cvss_pattern.search(file_content)

        if cvss_detect and cvss_detect.group("value") == "0.0":
            return

        # log_match = re.search(r'.*(log_message[\s]*\([^)]+\)[\s]*;)',
        #                       file_content, re.MULTILINE)
        log_match = re.search(
            r"log_message\s*\([\w:#\.&\-!,<>\[\]("
            r")\s\"+\'/\\\n]+\)\s*(;|;\s*(\n|#))",
            file_content,
            re.MULTILINE,
        )
        if log_match:
            yield LinterWarning(
                "The VT is using a log_message in a VT with a severity",
                file=nasl_file,
                plugin=self.name,
            )



# Copyright (C) 2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from pathlib import Path
from typing import Iterator

from troubadix.plugin import FileContentPlugin, LinterError, LinterResult


class CheckVariableAssignedInIf(FileContentPlugin):
    """This script checks the passed VT/Include if it is using
    a variable assignment within an if() call like e.g.:

    if( variable = "content" ) {}

    instead of:

    if( variable =~ "content" ) {}
    if( variable == "content" ) {}
    """

    name = "check_variable_assigned_in_if"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:
        """
        Args:
            file: The VT/Include that is going to be checked

        Returns:
        """
        # TO DO: Find a better way to parse if calls as this would
        # miss something like e.g.:
        #
        # if((foo =~ "bar" || bar =~ "foo") || foobar = "foo"){}
        #
        # nb: We can't use { as an ending delimiter as there could
        # be also something like e.g.:
        #
        # if((foo =~ "bar || bar =~ "foo") || foobar = "foo")
        #   bar = "foo"; (no ending {)
        matches = re.finditer(
            r"^\s*(if|}?\s*else if)\s*\(([^)]+)", file_content, re.MULTILINE
        )
        if matches is None:
            return

        lint_error = False
        output = (
            f"VT/Include '{nasl_file.name}' is using a variable assignment"
            " within an if() call in the following line(s):\n"
        )

        for match in matches:
            if match is not None and match.group(1) is not None:
                var_assign_match = re.search(
                    r"((if|}?\s*else if)\s*\(\s*?|\|{2}\s*|&{2}\s*)"
                    r'[a-zA-Z_][a-zA-Z0-9_]*\s*=\s*("|\'|TRUE|0|1)',
                    match.group(0),
                )
                if (
                    var_assign_match is not None
                    and var_assign_match.group(1) is not None
                ):
                    # nb: Can't be fixed because it would mean a change
                    # of a default behavior.
                    if (
                        "policy_file_checksums_win.nasl" in nasl_file.name
                        and "install = " in match.group(0)
                    ):
                        continue

                    output = f"{output} {match.group(0)}\n"
                    lint_error = True

        if lint_error:
            yield LinterError(
                output,
                file=nasl_file,
                plugin=self.name,
            )



# SPDX-License-Identifier: GPL-3.0-or-later
# SPDX-FileCopyrightText: 2024 Greenbone AG
import os
import sys
import tempfile
import unittest
from argparse import Namespace
from contextlib import redirect_stdout
from io import StringIO
from pathlib import Path
from unittest.mock import patch

from troubadix.standalone_plugins.file_extensions import (
    check_extensions,
    main,
    parse_args,
)


class TestFileExtensions(unittest.TestCase):
    def test_ok(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            # foo.nasl / foo.inc
            tempfile.mkstemp(dir=tmpdir, suffix=".nasl")
            tempfile.mkstemp(dir=tmpdir, suffix=".inc")

            # files in subfolders
            child_dir = tempfile.mkdtemp(dir=tmpdir)
            tempfile.mkstemp(dir=child_dir, suffix=".nasl")
            tempfile.mkstemp(dir=child_dir, suffix=".inc")

            parsed_args = Namespace(dir=Path(tmpdir), ignore_file=None)
            self.assertFalse(check_extensions(parsed_args))

    def test_exclusions(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            # manual exclusions
            new_dir = os.path.join(tmpdir, "common")
            os.mkdir(new_dir)
            with open(
                os.path.join(new_dir, "bad_rsa_ssh_host_keys.txt"),
                "w",
                encoding="utf-8",
            ):
                pass

            with open(os.path.join(tmpdir, "README.md"), "w", encoding="utf-8"):
                pass

            with open(
                os.path.join(tmpdir, "file_extension.ignore"),
                "a",
                encoding="utf-8",
            ) as f:
                exclusions = [
                    "README.md\n",
                    "common/bad_rsa_ssh_host_keys.txt\n",
                    "#ignore comment line",
                ]
                f.writelines(exclusions)

            parsed_args = Namespace(
                dir=Path(tmpdir),
                ignore_file=Path(tmpdir, "file_extension.ignore"),
            )
            actual = check_extensions(parsed_args)
            self.assertTrue(actual)
            self.assertEqual(actual, [Path(tmpdir, "file_extension.ignore")])

    def test_fail(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            # foo.nasl.nasl
            fp1 = Path(tempfile.mkstemp(dir=tmpdir, suffix=".nasl.nasl")[1])
            # foo.inc.inc
            fp2 = Path(tempfile.mkstemp(dir=tmpdir, suffix=".inc.inc")[1])
            # foo.nasl.inc
            fp3 = Path(tempfile.mkstemp(dir=tmpdir, suffix=".nasl.inc")[1])
            # foo.inc.nasl
            fp4 = Path(tempfile.mkstemp(dir=tmpdir, suffix=".inc.nasl")[1])
            child_dir = tempfile.mkdtemp(dir=tmpdir)
            # foo.bar
            fp5 = Path(tempfile.mkstemp(dir=child_dir, suffix=".bar")[1])
            # .foo
            fp6 = Path(tempfile.mkstemp(dir=child_dir, prefix=".")[1])
            # foo
            fp7 = Path(tempfile.mkstemp(dir=child_dir)[1])

            expected = {fp1, fp2, fp3, fp4, fp5, fp6, fp7}
            parsed_args = Namespace(dir=Path(tmpdir), ignore_file=None)
            actual = check_extensions(parsed_args)
            self.assertTrue(actual)
            self.assertEqual(set(actual), expected)

    def test_parse_args_ok(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            with open(
                os.path.join(tmpdir, "file_extensions.ignore"),
                "w",
                encoding="utf-8",
            ):
                pass
            test_args = [
                "prog",
                tmpdir,
                "--ignore-file",
                f"{tmpdir}/file_extensions.ignore",
            ]
            with patch.object(sys, "argv", test_args):
                args = parse_args()
                self.assertTrue(args)
                self.assertEqual(args.dir, Path(tmpdir))
                self.assertEqual(
                    args.ignore_file, Path(tmpdir, "file_extensions.ignore")
                )

    @patch("sys.stderr", new_callable=StringIO)
    def test_parse_args_no_dir(self, mock_stderr):
        test_args = ["prog", "not_real_dir"]
        with patch.object(sys, "argv", test_args):
            with self.assertRaises(SystemExit):
                parse_args()
            self.assertRegex(mock_stderr.getvalue(), "invalid directory_type")

    @patch("sys.stderr", new_callable=StringIO)
    def test_parse_args_no_file(self, mock_stderr):
        with tempfile.TemporaryDirectory() as tmpdir:
            test_args = ["prog", tmpdir, "--ignore-file", "not_real_file"]
            with patch.object(sys, "argv", test_args):
                with self.assertRaises(SystemExit):
                    parse_args()
                self.assertRegex(mock_stderr.getvalue(), "invalid file_type")

    def test_main_ok(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            tempfile.mkstemp(dir=tmpdir, suffix=".nasl")
            test_args = ["prog", tmpdir]
            with patch.object(sys, "argv", test_args):
                self.assertFalse(main())

    def test_main_fail(self):
        with StringIO() as buffer, redirect_stdout(buffer):
            with tempfile.TemporaryDirectory() as tmpdir:
                tempfile.mkstemp(dir=tmpdir)
                test_args = ["prog", tmpdir]
                with patch.object(sys, "argv", test_args):
                    self.assertTrue(main())


if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from itertools import chain
from pathlib import Path
from typing import Iterator

from troubadix.helper import SpecialScriptTag
from troubadix.helper.patterns import (
    _get_special_script_tag_pattern,
    _get_tag_pattern,
    get_xref_pattern,
)
from troubadix.plugin import FileContentPlugin, LinterError, LinterResult


class CheckScriptTagWhitespaces(FileContentPlugin):
    name = "check_script_tag_whitespaces"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:
        """
        Checks a given file content if a script tag value contains a leading or
        trailing whitespace character
        """
        if nasl_file.suffix == ".inc":
            return

        tag_matches = _get_tag_pattern(name=r".+?", flags=re.S).finditer(
            file_content
        )

        name_matches = _get_special_script_tag_pattern(
            name=SpecialScriptTag.NAME.value
        ).finditer(file_content)

        xref_matches = get_xref_pattern(name=r".+?", flags=re.S).finditer(
            file_content
        )

        matches = chain(tag_matches, name_matches, xref_matches)

        for match in matches:
            if re.match(r"^\s+.*", match.group("value")) or re.match(
                r".*\s+$", match.group("value"), flags=re.S
            ):
                yield LinterError(
                    f"{match.group(0)}: value contains a leading or"
                    " trailing whitespace character",
                    file=nasl_file,
                    plugin=self.name,
                )



# Copyright (C) 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from typing import List

from troubadix.standalone_plugins.changed_packages.package import (
    Direction,
    Package,
    Reasons,
)

from .marker import Marker


class DroppedArchitecture(Marker):
    @classmethod
    def mark(cls, missing_packages: List[Package], new_packages: List[Package]):
        for package in missing_packages:
            if not ":" in package.name:
                continue

            other_package = cls._find_package(
                Package(
                    package.name.split(":")[0],
                    package.version,
                    package.release,
                ),
                new_packages,
            )

            if not other_package:
                continue

            package.reasons[Reasons.DROPPED_ARCHITECTURE] = Direction.PASSIVE
            other_package.reasons[Reasons.DROPPED_ARCHITECTURE] = (
                Direction.ACTIVE
            )



# Copyright (C) 2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .helper import get_path_from_root, get_root, is_ignore_file, subprocess_cmd
from .patterns import (
    ScriptTag,
    SpecialScriptTag,
    get_common_tag_patterns,
    get_script_tag_pattern,
    get_special_script_tag_pattern,
)

# js: can we get this to utf-8 in future @scanner @feed?
CURRENT_ENCODING = "latin1"  # currently default



# SPDX-License-Identifier: GPL-3.0-or-later
# SPDX-FileCopyrightText: 2024 Greenbone AG

import re
from typing import Iterator

from troubadix.plugin import FilePlugin, LinterError, LinterResult


class CheckSpacesInFilename(FilePlugin):
    name = "check_spaces_in_filename"

    def run(self) -> Iterator[LinterResult]:
        if re.search(r"\s", self.context.nasl_file.name):
            yield LinterError(
                f"The VT {self.context.nasl_file}"
                " contains whitespace in the filename",
                file=self.context.nasl_file,
                plugin=self.name,
            )



# Copyright (C) 2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from typing import Iterator

from troubadix.plugin import FilePlugin, LinterError, LinterResult


class CheckDeprecatedFunctions(FilePlugin):
    name = "check_deprecated_functions"

    def run(self) -> Iterator[LinterResult]:
        """
        Following functions / description items are deprecated:
        script_summary()
        script_id()
        security_note()
        security_warning()
        security_hole()
        script_description()
        script_tag(name:"risk_factor", value:"SEVERITY");
        script_bugtraq_id()

        This script checks and reports if any of those are used

        Args:
            nasl_file: Name of the VT to be checked
        """
        deprecated_functions = {
            'script_summary();, use script_tag(name:"summary", value:""); '
            "instead": r"script_summary\s*\([^)]*\);",
            "script_id();, use script_oid(); with "
            "the full OID instead": r"script_id\s*\([0-9]+\);",
            "security_note();": r"security_note\s*\([^)]*\);",
            "security_warning();": r"security_warning\s*\([^)]*\);",
            "security_hole();": r"security_hole\s*\([^)]*\);",
            "script_description();": r"script_description\s*\([^)]*\);",
            'script_tag(name:"risk_factor", value:'
            '"SEVERITY");': r'script_tag\s*\(\s*name:\s*"risk_factor"[^)]*\);',
            "script_bugtraq_id();": r"script_bugtraq_id\s*\([^)]*\);",
        }

        for description, pattern in deprecated_functions.items():
            if re.search(pattern, self.context.file_content, re.MULTILINE):
                yield LinterError(
                    "Found a deprecated function call / description item: "
                    f"{description}",
                    file=self.context.nasl_file,
                    plugin=self.name,
                )



# SPDX-License-Identifier: GPL-3.0-or-later
# SPDX-FileCopyrightText: 2024 Greenbone AG

import re
import sys
from argparse import ArgumentParser, Namespace
from pathlib import Path
from typing import List


def directory_type(string: str) -> Path:
    directory_path = Path(string)
    if not directory_path.is_dir():
        raise ValueError(f"{string} is not a directory.")
    return directory_path


def file_type(string: str) -> Path:
    file_path = Path(string)
    if not file_path.is_file():
        raise ValueError(f"{string} is not a file.")
    return file_path


def parse_args() -> Namespace:
    parser = ArgumentParser(
        description="Check for files with unwanted file extensions",
    )
    parser.add_argument(
        "dir",
        type=directory_type,
        help="directory that should be linted",
    )
    parser.add_argument(
        "--ignore-file", type=file_type, help="path to ignore file"
    )
    return parser.parse_args()


def create_exclusions(ignore_file: Path) -> set[Path]:
    if ignore_file is None:
        return set()

    with open(ignore_file, "r", encoding="utf-8") as file:
        return {
            Path(line.strip()) for line in file if not re.match(r"^\s*#", line)
        }


def check_extensions(args: Namespace) -> List[Path]:
    """This script checks for any non .nasl or .inc file."""
    unwanted_files: List[Path] = []
    allowed_extensions = [".inc", ".nasl"]
    exclusions = create_exclusions(args.ignore_file)

    for item in args.dir.rglob("*"):
        if not item.is_file():
            continue

        relative_path = item.relative_to(args.dir)
        if relative_path in exclusions:
            continue

        # foo.inc.inc / foo.nasl.nasl
        # foo.inc.nasl / foo.nasl.inc
        if len(item.suffixes) > 1:
            unwanted_files.append(item)

        # foo / foo.bar
        if item.suffix not in allowed_extensions:
            unwanted_files.append(item)

    return unwanted_files


def main() -> int:
    if unwanted_files := check_extensions(parse_args()):
        print(
            f"{len(unwanted_files)} "
            "Files with unwanted file extension were found:"
        )
        for file in unwanted_files:
            print(file)
        return 1

    return 0


if __name__ == "__main__":
    sys.exit(main())



# Copyright (C) 2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from pathlib import Path
from typing import Iterator

from troubadix.plugin import FileContentPlugin, LinterError, LinterResult

VALUES = (
    "Workaround",
    "Mitigation",
    "VendorFix",
    "NoneAvailable",
    "WillNotFix",
)


class CheckSolutionType(FileContentPlugin):
    """
    This script checks the passed VT for the existence/format of its
    solution_type with the help of regular expression.An error will be thrown
    if the VT does not contain a solution_type at all or of the solution_type
    contains an invalid value.
    """

    name = "check_solution_type"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:
        if (
            nasl_file.suffix == ".inc"
            or "# troubadix: disable=template_nd_test_files_fps" in file_content
        ):
            return

        has_severity = True
        cvss_detect = re.search(
            r"script_tag\s*\(name\s*:\s*\"cvss_base\","
            r"\s*value:\s*\"(\d{1,2}\.\d)\"\)",
            file_content,
        )
        if cvss_detect is not None and cvss_detect.group(1) == "0.0":
            has_severity = False

        match = re.search(
            r"script_tag\s*\(name\s*:\s*\"(solution_type)\"\s*,"
            r"\s*value\s*:\s*\"([a-zA-Z\s]+)\"\s*\)\s*;",
            file_content,
        )

        if has_severity:
            if match is None or match.group(1) is None:
                yield LinterError(
                    "VT does not contain a solution_type",
                    file=nasl_file,
                    plugin=self.name,
                )
        if match is not None and match.group(2) not in VALUES:
            yield LinterError(
                "VT does not contain a valid solution_type 'value'",
                file=nasl_file,
                plugin=self.name,
            )



# SPDX-License-Identifier: GPL-3.0-or-later
# SPDX-FileCopyrightText: 2024 Greenbone AG

from pathlib import Path

from tests.plugins import PluginTestCase
from troubadix.plugin import LinterError
from troubadix.plugins.spaces_in_filename import CheckSpacesInFilename


class TestSpacesInFilename(PluginTestCase):
    def test_ok(self):
        nasl_file = Path(__file__).parent / "foo.nasl"
        fake_context = self.create_file_plugin_context(nasl_file=nasl_file)
        plugin = CheckSpacesInFilename(fake_context)
        results = list(plugin.run())
        self.assertEqual(len(results), 0)

    def test_fail(self):
        nasl_file = Path(__file__).parent / "foo bar.nasl"
        fake_context = self.create_file_plugin_context(nasl_file=nasl_file)
        plugin = CheckSpacesInFilename(fake_context)
        results = list(plugin.run())
        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            results[0].message,
            f"The VT {nasl_file} contains whitespace in the filename",
        )



# Copyright (C) 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from unittest import TestCase

from troubadix.standalone_plugins.changed_packages.marker import AddedUdeb
from troubadix.standalone_plugins.changed_packages.package import (
    Direction,
    Package,
    Reasons,
)


class AddedUdebTestCase(TestCase):
    def test_mark(self):
        new_packages = [
            Package("foo", "1:1.2.3", "DEB11"),
            Package("bar-udeb", "4.5-6", "DEB10"),
            Package("baz-udeb", "2:1.3.3.7", "DEB11"),
        ]

        expected_new_packages = [
            Package(
                "foo",
                "1:1.2.3",
                "DEB11",
                {Reasons.ADDED_UDEB: Direction.ACTIVE},
            ),
            Package(
                "bar", "4.5-6", "DEB11", {Reasons.ADDED_UDEB: Direction.ACTIVE}
            ),
            Package(
                "baz",
                "2:1.3.3.7",
                "DEB11",
                {Reasons.ADDED_UDEB: Direction.ACTIVE},
            ),
        ]

        AddedUdeb.mark(new_packages)

        self.assertEqual(expected_new_packages, expected_new_packages)



# Copyright (C) 2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from pathlib import Path
from typing import Iterator

from troubadix.helper.patterns import _get_tag_pattern
from troubadix.plugin import FileContentPlugin, LinterError, LinterResult


class CheckValidScriptTagNames(FileContentPlugin):
    """This plugin checks if the 'name' parameter of the script tags having the
    following form:

    - script_tag(name:"", value:"");

    is one of the following allowed ones:

    - script_tag(name:"solution", value:"");
    - script_tag(name:"solution_type", value:"");
    - script_tag(name:"qod_type", value:"");
    - script_tag(name:"cvss_base", value:"");
    - script_tag(name:"cvss_base_vector", value:"");
    - script_tag(name:"summary", value:"");
    - script_tag(name:"last_modification", value:"");
    - script_tag(name:"insight", value:"");
    - script_tag(name:"affected", value:"");
    - script_tag(name:"creation_date", value:"");
    - script_tag(name:"vuldetect", value:"");
    - script_tag(name:"impact", value:"");
    - script_tag(name:"deprecated", value:"");
    - script_tag(name:"qod", value:"");
    - script_tag(name:"severity_vector", value:"");
    - script_tag(name:"severity_origin", value:"");
    - script_tag(name:"severity_date", value:"");
    nb: The following is not fully implemented in GVM yet (further
    implementation "on hold")
    - script_tag(name:"solution_method", value:"");
    """

    name = "check_valid_script_tag_names"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:
        """
        Args:
            nasl_file: The VT that is going to be checked
        """
        if nasl_file.suffix == ".inc":
            return

        allowed_script_tag_names = [
            "solution",
            "solution_type",
            "qod_type",
            "cvss_base",
            "cvss_base_vector",
            "summary",
            "last_modification",
            "insight",
            "affected",
            "creation_date",
            "vuldetect",
            "impact",
            "deprecated",
            "qod",
            "severity_vector",
            "severity_origin",
            "severity_date",
            "solution_method",
        ]

        matches = _get_tag_pattern(name=r".+?", flags=re.S).finditer(
            file_content
        )

        if matches:
            for match in matches:
                if match.group("name") not in allowed_script_tag_names:
                    yield LinterError(
                        f"The script_tag name '{match.group('name')}' "
                        "is not allowed.",
                        file=nasl_file,
                        plugin=self.name,
                    )



# Copyright (C) 2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from pathlib import Path
from typing import Iterator

from troubadix.helper import CURRENT_ENCODING
from troubadix.plugin import (
    FileContentPlugin,
    LinterError,
    LinterFix,
    LinterResult,
)

CORRECT_COPYRIGHT_PHRASE = (
    "# Some text descriptions might be excerpted from (a) referenced\n"
    "# source(s), and are Copyright (C) by the respective right holder(s)."
)


class CheckCopyrightText(FileContentPlugin):
    name = "check_copyright_text"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:
        """This plugin will report any occurrence of the following outdated text
        pattern:

        # Text descriptions are largely excerpted from the referenced
        # advisory, and are Copyright (C) of their respective author(s)

        or:

        # Text descriptions are largely excerpted from the referenced
        # advisory, and are Copyright (C) the respective author(s)

        or:

        # Text descriptions are largely excerpted from the referenced
        # advisory, and are Copyright (C) the respective author(s)

        or:

        # Some text descriptions might be excerpted from the referenced
        # advisories, and are Copyright (C) by the respective right holder(s)

        which should be the following from now on:

        # Some text descriptions might be excerpted from (a) referenced
        # source(s), and are Copyright (C) by the respective right holder(s).
        """
        self.new_file_content = None

        if nasl_file.suffix == ".inc":
            return

        match = re.search(
            r"^# (Text descriptions are largely excerpted from the referenced"
            r"\n# advisory, and are Copyright \([cC]\) (of )?(the|their) resp"
            r"ective author\(s\)|Some text descriptions might be excerpted from"
            r" the referenced\n# advisories, and are Copyright \(C\) by the "
            r"respective right holder\(s\))",
            file_content,
            re.MULTILINE,
        )
        if match:
            self.new_file_content = file_content.replace(
                match.group(0),
                CORRECT_COPYRIGHT_PHRASE,
            )

            yield LinterError(
                "The VT is using an incorrect copyright statement.",
                file=nasl_file,
                plugin=self.name,
            )

    def fix(self) -> Iterator[LinterResult]:
        if not self.new_file_content:
            return

        nasl_file = self.context.nasl_file
        nasl_file.write_text(
            data=self.new_file_content, encoding=CURRENT_ENCODING
        )

        yield LinterFix(
            f"The copyright statement has been updated to "
            f"{CORRECT_COPYRIGHT_PHRASE}",
            file=nasl_file,
            plugin=self.name,
        )
